/* ------------------------------ npaste.c ----------------------------	*/
/* This program loads multiple floppy sized files to OutFile.		*/
/* It is similar to paste.c but it gets the file name from the floppy.	*/
/*									*/
/*	usage:				to:		from:		*/
/*	npaste           		file.zip	a:file.000	*/
/*	npaste .arj           		file.arj	a:file.000	*/
/*	npaste d:file.lzh		d:file.lzh	a:file.000	*/
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	Sep 1994						*/
/* Version:	1.1							*/
/*									*/
/* This program is released into the public domain. Use it at		*/
/* your own risk.							*/
/*									*/
/* --------------------------------------------------------------------	*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <dos.h>

static char inbuf[4096], outbuf[4096];
char in_name[80];   /* input file */
static char rec[1024];

static void
usage ()
{
   fprintf (stderr, "npaste v1.1 Sep/1994 by Eyal Lebedinsky\n");
   fprintf (stderr, "usage: npaste [OutFile -v -b]\n");
   fprintf (stderr, "\tOutFile full name of outfile or extension\n");
   fprintf (stderr, "\t-v      verbose\n");
   fprintf (stderr, "\t-dx     read from drive X:\n");
   exit (4);
}

void
main (argc, argv)
int argc;
char *argv[];
{
	FILE	*fin, *fout;
	long	i = 0, tot_size = 0;
	int	parm, verbose, drive, rec_size, flag, j;
	char	out_name[80], *disk_no, *p, *q;
	struct find_t d;

	_fmode = O_BINARY;

	drive = 'A';
	verbose = 0;
	parm = 0;

	for (j = 1; j < argc; ++j) {
		if (argv[j][0] != '-') {
			if (parm) {
				fprintf (stderr, "only one file expected\n");
				usage ();
			}
			parm = j;
			continue;
		}
		switch (toupper (argv[j][1])) {
		case 'D':
			drive = toupper (argv[j][2]);
			break;

		case 'V':
			verbose = 1;
			break;
		}
	}

	strcpy (in_name, "A:*.000");
	in_name[0] = drive;

	if (_dos_findfirst (in_name, _A_NORMAL, &d)) {
		fprintf (stderr, "\nCan not read directory %s\n", in_name);
		exit (4);
	}

	strcpy (in_name+2, d.name);		/* A:filename.000 */

	strcpy (out_name, d.name);		/* filename.000 */
	out_name[strlen (out_name) - 4] = '\0';	/* filename */

	if (parm == 0)				/* no file name */
		strcat (out_name, ".zip");
	else if (argv[parm][0] == '.')		/* only extension */
		strcat (out_name, argv[parm]);
	else					/* full file name */
		strcpy (out_name, argv[parm]);

	fprintf (stderr, "pasting from %s to %s\n",
		in_name, out_name);

	fprintf (stderr, "\nPress Return when ready (^C to stop)...");
	(void) getchar ();

	if ((fout = fopen (out_name, "wb")) == NULL) {
		fprintf (stderr, "\nCan not open %s\n", out_name);
		exit (4);
	}
	if (setvbuf (fout, outbuf, _IOFBF, 4096)) {
		fprintf (stderr, "output buffering failed\n");
		exit (4);
	}

	disk_no = in_name + strlen (in_name) - 3;
	
	flag = 0;		/* do not prompt first time */
	for (;;) {

		if (flag) {
			fprintf (stderr, "\a\nPut diskette %s in drive %c:",
				in_name, drive);
			fprintf (stderr, "\nPress Return when ready (* if end of set)...");
			if (getchar() == '*')
				break;
		}
		
		if ((fin = fopen (in_name, "rb")) == NULL) {
			if (flag)
				fprintf (stderr, "\nError opening input file\n");
			flag = 1;
			continue;
		}

		if (setvbuf (fin, inbuf, _IOFBF, 4096)) {
			fprintf (stderr, "input buffering failed\n");
			exit (4);
		}

		if (disk_no[2] == '9') {
			disk_no[2] = '0';
			if (disk_no[1] == '9') {
				disk_no[1] = '0';
				if (disk_no[0] == '9') {
					fprintf (stderr, "Too many disks used. Aborting\n");
					exit (4);
				} else
					++disk_no[0];
			} else
				++disk_no[1];
		} else
			++disk_no[2];


		/* read records from input files until end of file	*/

		flag = 1;
		for (;;) {

			if ((rec_size = fread (rec, 1, 1024, fin)) == 0)
				break;
				
			flag = 0;
			
			fwrite (rec, rec_size, 1, fout);
			tot_size += rec_size;

			if (verbose)
				fprintf (stderr, "paste %luk bytes\r", ++i);
		}
		fclose (fin);
		if (flag)
			break;		/* file was empty - done */
		flag = 0;
	}

	fclose (fout);

	fprintf (stderr, "paste: %lu bytes written to %s.\n",
		tot_size, out_name);

	exit (0);
}
